#include <vcl.h>
#pragma hdrstop

#include "MultiCams_.h"
#pragma package(smart_init)
#pragma link "MoDetect"
#pragma link "MoDetect"
#pragma resource "*.dfm"

TForm1 *Form1;

__fastcall TForm1::TForm1(TComponent* Owner)
        : TForm(Owner)
{
}
//------------------------------------------------------------------------------

void __fastcall TForm1::SetupGrid (TMotionDetector *MotionDetector, TMemo *Memo, TGroupBox *GroupBox)
{
   int xCells, yCells;

   xCells = StrToIntDef (Edit5->Text, 8) ;
   yCells = StrToIntDef (Edit5->Text, 8) ;

   if (MotionDetector->SetGridSize (xCells, yCells)) {
      Memo->Text = MotionDetector->Get2DTextGrid();
      Memo->Width = MotionDetector->GridXCount() * 14;
      Memo->Height = MotionDetector->GridYCount() * 15 + 3;
   }
   GroupBox->Width = Memo->Left + Memo->Width + 8; // we adapt the GroupBox size to the grid's size
   GroupBox->Height = Memo->Height + 24;

   if (GroupBox->Height < 137) {
      GroupBox->Height = 137;                    // however we avoid having the GroupBox smaller than the video preview
   }
}
//------------------------------------------------------------------------------

void __fastcall TForm1::SetupGrids (void)
{
   SetupGrid (MotionDetector1, Memo1, GroupBox1);
   SetupGrid (MotionDetector2, Memo2, GroupBox2);
   SetupGrid (MotionDetector3, Memo3, GroupBox3);
   SetupGrid (MotionDetector4, Memo4, GroupBox4);
   GroupBox2->Top = GroupBox1->Top + GroupBox1->Height + 4;
   GroupBox3->Top = GroupBox2->Top + GroupBox2->Height + 4;
   GroupBox4->Top = GroupBox3->Top + GroupBox3->Height + 4;
   Form1->Width = GroupBox1->Left + GroupBox1->Width + 14;
   Form1->Height = GroupBox4->Top + GroupBox4->Height + 30;
}
//------------------------------------------------------------------------------

void __fastcall TForm1::SetupTVideoGrabber (TVideoGrabber* VideoGrabber, TMotionDetector *MotionDetector)
{
   VideoGrabber->UseNearestVideoSize (160, 120, false); // we select the video size nearest to 160x120 (higher sizes increase CPU load)

   // THE 5 PROPERTIES BELOW CAN BE SET FROM THE OBJECT INSPECTOR

   VideoGrabber->MotionDetector = MotionDetector;  // connects the TMotionDetector component with the TVideoGrabber component
   VideoGrabber->AutoSize = False;                 // the video preview will be stretched to the component size
   VideoGrabber->BurstMode = True;                 // enabled frame capture
   VideoGrabber->BurstCount = 0;                   // not to stop frame capture
   VideoGrabber->BurstInterval = 0;                // not to skip frames
}
//------------------------------------------------------------------------------

void __fastcall TForm1::FormCreate(TObject *Sender)
{
   SetupGrids();                        // let's setup the grid sizes...

   SetupTVideoGrabber (VideoGrabber1, MotionDetector1); // ... and the TVideoGrabber components
   SetupTVideoGrabber (VideoGrabber2, MotionDetector2);
   SetupTVideoGrabber (VideoGrabber3, MotionDetector3);
   SetupTVideoGrabber (VideoGrabber4, MotionDetector4);
}
//------------------------------------------------------------------------------

void __fastcall TForm1::UpDown1Click(TObject *Sender, TUDBtnType Button)
{
   SetupGrids();
}
//------------------------------------------------------------------------------

void __fastcall TForm1::UpDown2Click(TObject *Sender, TUDBtnType Button)
{
   MotionDetector1->SetGlobalSensitivity (StrToIntDef (Edit6->Text, 8));
   MotionDetector2->SetGlobalSensitivity (StrToIntDef (Edit6->Text, 8));
   MotionDetector3->SetGlobalSensitivity (StrToIntDef (Edit6->Text, 8));
   MotionDetector4->SetGlobalSensitivity (StrToIntDef (Edit6->Text, 8));
   SetupGrids(); // to refresh the grid sensitivity display
}
//------------------------------------------------------------------------------

void __fastcall TForm1::CheckBox1Click(TObject *Sender)
{
   if (CheckBox1->Checked) {
      VideoGrabber1->StartPreview();
   }
   else {
      VideoGrabber1->StopPreview();
   }
}
//---------------------------------------------------------------------------

void __fastcall TForm1::CheckBox2Click(TObject *Sender)
{
   if (CheckBox2->Checked) {
      VideoGrabber2->StartPreview();
   }
   else {
      VideoGrabber2->StopPreview();
   }
}
//---------------------------------------------------------------------------

void __fastcall TForm1::CheckBox3Click(TObject *Sender)
{
   if (CheckBox3->Checked) {
      VideoGrabber3->StartPreview();
   }
   else {
      VideoGrabber3->StopPreview();
   }
}
//---------------------------------------------------------------------------

void __fastcall TForm1::CheckBox4Click(TObject *Sender)
{
   if (CheckBox4->Checked) {
      VideoGrabber4->StartPreview();
   }
   else {
      VideoGrabber4->StopPreview();
   }
}
//---------------------------------------------------------------------------

void __fastcall TForm1::CheckBox5Click(TObject *Sender)
{
   MotionDetector1->ReduceVideoNoise = CheckBox5->Checked;
   MotionDetector2->ReduceVideoNoise = CheckBox5->Checked;
   MotionDetector3->ReduceVideoNoise = CheckBox5->Checked;
   MotionDetector4->ReduceVideoNoise = CheckBox5->Checked;
}
//------------------------------------------------------------------------------

void __fastcall TForm1::ProcessMotion (bool MotionDetected, TMotionDetector *MotionDetector, TShape *Shape, TMemo *Memo, TEdit *Edit, TImage *Image)
{
   if (MotionDetected) {
      Shape->Brush->Color = clLime;
      Memo->Text = MotionDetector->Get2DTextMotion();
      Edit->Text = FloatToStr (MotionDetector->GlobalMotionRatio());
      Image->Picture->Bitmap->Assign (MotionDetector->LastFrame);
   }
   else {
      Shape->Brush->Color = clGreen;
      Memo->Text = "";
      Edit->Text = "";
   }
}
//------------------------------------------------------------------------------

void __fastcall TForm1::MotionDetector1MotionDetected(TObject *Sender, double GlobalMotionRatio)
{
   ProcessMotion (true, MotionDetector1, Shape1, Memo1, Edit1, Image1);
}
//---------------------------------------------------------------------------

void __fastcall TForm1::MotionDetector2MotionDetected(TObject *Sender, double GlobalMotionRatio)
{
   ProcessMotion (true, MotionDetector2, Shape2, Memo2, Edit2, Image2);
}
//---------------------------------------------------------------------------

void __fastcall TForm1::MotionDetector3MotionDetected(TObject *Sender, double GlobalMotionRatio)
{
   ProcessMotion (true, MotionDetector3, Shape3, Memo3, Edit3, Image3);
}
//---------------------------------------------------------------------------

void __fastcall TForm1::MotionDetector4MotionDetected(TObject *Sender, double GlobalMotionRatio)
{
   ProcessMotion (true, MotionDetector4, Shape4, Memo4, Edit4, Image4);
}
//---------------------------------------------------------------------------

void __fastcall TForm1::MotionDetector1MotionNotDetected(TObject *Sender)
{
   ProcessMotion (false, MotionDetector1, Shape1, Memo1, Edit1, Image1);
}
//---------------------------------------------------------------------------

void __fastcall TForm1::MotionDetector2MotionNotDetected(TObject *Sender)
{
   ProcessMotion (false, MotionDetector2, Shape2, Memo2, Edit2, Image2);
}
//---------------------------------------------------------------------------

void __fastcall TForm1::MotionDetector3MotionNotDetected(TObject *Sender)
{
   ProcessMotion (false, MotionDetector3, Shape3, Memo3, Edit3, Image3);
}
//---------------------------------------------------------------------------

void __fastcall TForm1::MotionDetector4MotionNotDetected(TObject *Sender)
{
   ProcessMotion (false, MotionDetector4, Shape4, Memo4, Edit4, Image4);
}
//---------------------------------------------------------------------------

